/** 
	A program in Qt
	that use OpenGL drawing primitives
	to draw a Cylinder
	File: cubes.cpp

	Fraga 28/11/2000
	Fraga 21/10/2003
**/
#include <qpainter.h>
#include <stdio.h>
#include <stdlib.h>
#include "cylinder.h"

Cylinder::Cylinder( QWidget *parent, const char *name )
		: QGLWidget( parent, name )
{
	signo = 1;	

	angle = 0.0;
	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	n = 12;
}


void Cylinder::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cylinder::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Cylinder::resizeGL( int w, int h )
{
	double aspect=(double)w/(double)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if ( w <= h ) 
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho ( -aspect, aspect, -1.0, 1.0, -1.0, 1.0);

	emit changeSize ( );
}

QSizePolicy Cylinder::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** 
	Function that generate a cylinder with origin in (cx,cy,cz)
	with size value for its side
**/
float **
Cylinder::getCylinder ( float l, float r, float cx, float cy, float cz ) 
{
	float ** pcyl;

	float theta;
	int i, j;
	float x, z;

	pcyl = (float **) malloc( 3*sizeof(float *));
	if ( pcyl == NULL ) return NULL;
 
	pcyl[0] = (float *)malloc ( n*3*sizeof(float) );
	if ( !pcyl[0] ){
		free( pcyl); return (NULL);
	}
	pcyl[1] = (float *)malloc ( n*3*sizeof(float) );
	if ( !pcyl[1] ){
		free(pcyl[0]); free( pcyl); return (NULL);
	}
	pcyl[2] = (float *)malloc ( n*3*sizeof(float) );
	if ( !pcyl[2] ){
		free(pcyl[1]); free(pcyl[0]); free( pcyl); return (NULL);
	}

	theta = 2*M_PI/n;

	for( i=0; i<n; i++ ) {
		x = r*cos( i*theta );
		z = r*sin( i*theta );

		j = i*3;

		// primero la tapa
		pcyl[0][j] = cx + x;
		pcyl[0][j+1] = cy + l/2.0;
		pcyl[0][j+2] = cz + z;
		
		// luego la base
		pcyl[1][j] = cx + x;
		pcyl[1][j+1] = cy + -l/2.0;
		pcyl[1][j+2] = cz + z;


		// las normales a cada vrtice
		x /= r;
		z /= r;
		pcyl[2][j] = x;
		pcyl[2][j+1] = 0.0;
		pcyl[2][j+1] = z;
/**
		// las normales por cara
		x = cos( i*theta + theta/2.0 );
		z = sin( i*theta + theta/2.0 );
		pcyl[2][j] = x;
		pcyl[2][j+1] = 0.0;
		pcyl[2][j+1] = z;
**/
	}

	return ( pcyl );
}

void Cylinder::showCylinder ( float **c )
{
	int i, j, k;

	for (i = 0; i < n; i++) {
		glBegin(GL_QUADS);

		j = i*3;
		k = 3*((i+1)%n);	

		glNormal3fv( &c[2][j]);
		glVertex3fv( &c[0][j] );

		glNormal3fv( &c[2][k]);
		glVertex3fv( &c[0][k] );
		glVertex3fv( &c[1][k] );

		glNormal3fv( &c[2][j]);
		glVertex3fv( &c[1][j] );

		glEnd();
	}
}

void Cylinder::paintGL(void)
{
	#define SINGLEINS  0

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	// glScalef ( 0.7, 0.7, 0.7);	  /* modeling transformation */ 

	showCylinder( c1 );

	// Dibujo los ejes
	glBegin(GL_LINES);
		// x en rojo
		glColor3ub( 255, 0, 0 );
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( 1.0, 0.0, 0.0 );
		
		// y en verde
		glColor3ub( 0, 255, 0 );
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( 0.0, 1.0, 0.0 );

		// z en azul
		glColor3ub( 0, 0, 255 );
		glVertex3f( 0.0, 0.0, 0.0 );
		glVertex3f( 0.0, 0.0, 1.0 );
	glEnd();

	swapBuffers();
}

void Cylinder::initializeGL( ) 
{
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 50.0 };
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat white_light[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat lmodel_ambient[] = { 0.1, 0.1, 0.1, 1.0 };

	glClearColor ( 0.0, 0.0, 0.0, 0.0);	// Background to black

	glShadeModel( GL_SMOOTH );
	// glShadeModel( GL_FLAT );
	glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
	glMaterialfv( GL_FRONT, GL_SHININESS, mat_shininess );

	glLightfv( GL_LIGHT0, GL_POSITION, light_position );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, white_light );
	glLightfv( GL_LIGHT0, GL_SPECULAR, white_light );

	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0,  1.0);

	c1 = getCylinder ( 0.8, 0.4, 0.0, 0.0, 0.0 );

	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_DEPTH_TEST);
}

void Cylinder::keyPressEvent( QKeyEvent *e )
{
	switch ( e->key() ) {
		case Key_R: // Reset
			xrot = yrot = 0.0;	
			angle=0.0;
			updateGL( );
			break;
		default:
			return;
	}
	e->accept( );
}


