/** 
	A simple program to rendering a lit sphere
	From book:
	OpenGL Programming Guide
	File: sphere.cpp

	Fraga 02/12/2005
**/
#include <qpainter.h>
#include <ctype.h>
#include "sphere.h"

Sphere::Sphere( QWidget *parent, const char *name )
        : QGLWidget( parent, name )
//  : QGLWidget( QGLFormat( SingleBuffer|NoDepthBuffer ), parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

}

Sphere::~Sphere( )
{
	gluDeleteQuadric( sphere );
}


void Sphere::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Sphere::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Sphere::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Sphere::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho ( -1.5, 1.5, -1.5/aspect, 1.5/aspect, -10.0, 10.0);
	else
		glOrtho (-1.5*aspect, 1.5*aspect, -1.5, 1.5, -10.0, 10.0);
	
	emit changeSize ( );
}

QSizePolicy Sphere::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Sphere::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	glCallList( slist );

	swapBuffers( );
}

void Sphere::initializeGL( ) 
{
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 50.0 };
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat white_light[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat lmodel_ambient[] = { 0.1, 0.1, 0.1, 1.0 };

	glClearColor ( 0.0, 0.0, 0.0, 0.0);	// Background to black

	glShadeModel( GL_SMOOTH );
	// glShadeModel( GL_FLAT );
	glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
	glMaterialfv( GL_FRONT, GL_SHININESS, mat_shininess );

	glLightfv( GL_LIGHT0, GL_POSITION, light_position );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, white_light );
	glLightfv( GL_LIGHT0, GL_SPECULAR, white_light );

	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );

	slist = glGenLists(1);
	sphere = gluNewQuadric();
	gluQuadricDrawStyle( sphere, GLU_FILL );
	gluQuadricNormals( sphere, GLU_SMOOTH );
	// gluQuadricNormals( sphere, GLU_FLAT );
	glNewList( slist, GL_COMPILE );
		gluSphere( sphere, 1.0, 20, 16 );
	glEndList();
	

	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_DEPTH_TEST);


}

